# Sserver Devcommands

This client side mod allows devcommands and utilities for server admins.

Some features and commands require also installing the mod on the server (event, randomevent, resetkeys, skiptime, sleep, stopevent).

# Manual Installation:

1. Install the [BepInExPack Valheim](https://valheim.thunderstore.io/package/denikson/BepInExPack_Valheim)
2. Download the latest zip.
3. Extract it in the \<GameDirectory\>\BepInEx\plugins\ folder.
4. Add your steamID64 to adminlist.txt (if not already).

Check [wiki](https://valheim.fandom.com/wiki/Console_Commands) for available commands and how to use them.

# Features

- Console is enabled without having to set the start parameter.
- Cheat commands can also be used from the chat window (with autocomplete).
- Autocomplete works for every parameter, always providing some information.
- Multiple commands can be executed at the same time (when separated with `;`).
- New commands can be created with `alias` command.
- Modifier keys work when binding commands to keys.
- `devcommands` is used automatically (if the admin check passes).
- `debugmode`, `fly`, `ghost`, `god`, `nocost` and other commands can be configured to be used automatically.
- God mode removes stamina usage, knockback and staggering.
- Ghost mode makes you invisible also to other players.
- Minor tweaks to existing commands, some new commands and other useful admin features.

## Improved key bindings

Keybindings now work with modifier keys ([key codes](https://docs.unity3d.com/ScriptReference/KeyCode.html)).

- `bind [keycode] [value] [keys=]`: Adds a new key binding with modifier keys.
	- `bind j god`: Toggles god mode when pressing J.
	- `bind j god keys=leftalt`: Toggles god mode when pressing J while left alt is down.
	- `bind j god keys=-leftalt`: Toggles god mode when pressing J while left alt is not down.
	- `bind j god keys=leftalt,h`: Toggles god mode when pressing J while both left alt and h are down.	

After removing this mod, these binds very likely stop working or lead to unexpected behavior. Recommended to clear all binds with the `resetbinds` command.

## Command aliasing

New commands can be created to shorten command names or to set parameter values.

This is intended to be used with other mods that add more complex commands than in the base game.

- `alias [name] [value]`: Adds a new command alias.
- `alias`: Prints all aliases.
- `alias [name]`: Removes the given alias.

Examples:

- `alias dm debugmode`: Adds a new command `dm` as a shorter version of `debugmode`.
- `alias spawn5 spawn $ 5 $`: Adds a new command `spawn5` with the spawn amount fixed at 5.
- `alias maxskill raiseskill $ 100`: Adds a new command `skill_max` that raises the given skill to max level.
- `alias resetskill raiseskill $ -100`: Adds a new command `skill_reset` that resets the given skill.
- `alias cheat debugmode;nocost;fly`: Adds a new command `cheat` to quickly toggle cheats (if you don't want to use the config).

## Enhanced commands

- `pos [player name]` allows getting the position of any player.
	- `pos`: Returns your position.
	- `pos jay`: Returns the position of a player named Jay,Heyjay or whatever is the closest match.
- `event [event] [x] [z]` allows setting the event coordinates.
	- `event army_eikthyr`: Starts an event at your position.
	- `event army_eikthyr 100 -100`: Starts an event at coordinates 100,-100.
- `devcommands` includes an admin check to allow using on servers.
- `dev_config [value]` toggles settings.
- `dev_server_config [value]` toggles settings on the server.
- `redo` restores an action added to the undo/redo manager.
- `resolution` prints the screen properties.
- `resolution [mode] [width] [height] [refresh rate]` sets the screen properties.
	- `resolution exclusive`: Sets to full screen with the maximum supported resolution and refresh rate.
	- `resolution max`: Sets to maximized window with the maximum supported resolution.
	- `resolution full`: Sets to full screen window with the maximum supported resolution.
	- `resolution window 1920 1080`: Sets to windowed with HD resolution.
- `search [term] [max_lines=5]` allows searching the object ID list.
	- `search wolf`: Prints all object IDs that contain word "wolf".
	- `search fx_ 10`: Prints all object IDs that contain word "fx_" on up to 10 lines.
- `tutorialtoggle [value]` allows directly setting the value.
  - `dev_config auto_exec tutorialtoggle 0`: Automatically disables tutorials.
- `undo` reverts an action added to the undo/redo manager.

## Enhanced map

The large map shows coordinates of the cursor when hovered. This can be useful for any commands that require coordinates.

If this mod is also installed on the server, admins can also receive position of players who have set their position as private (disabled by default).

These players are shown on the map with a ticked off icon and will also be available for the `pos` command.

# Configuration

Three ways to edit the settings:

- Use `dev_config` and `dev_server_config` commands to instantly toggle values.
- Use the [Configuration manager](https://github.com/BepInEx/BepInEx.ConfigurationManager/releases/tag/v16.4) if installed to instantly toggle values for the client.
- Manually edit the `valheim.jerekuusela.dev.cfg` in the config folder (requires restarting the client / server).

Recommended way is to use the commands since you can configure the server and also `bind` them to keys.

## General

- Automatic debug mode (default `false`, key: `auto_debugmode`): Automatically turns debug mode on/off when devcommands are enabled or disabled.
- Automatic devcommands (default `true`, key: `auto_devcommands`): Automatically tries to enable devcommands when joining servers.
- Automatic fly mode (default `false`, key: `auto_fly`): Automatically turns fly mode on/off when devcommands are enabled or disabled. Requires automatic debug mode.
- Automatic ghost mode (default `false`, key: `auto_ghost`): Automatically turns ghost mode on/off when devcommands are enabled or disabled.
- Automatic god mode (default `false`, key: `auto_god`): Automatically turns god mode on/off when devcommands are enabled or disabled.
- Automatic no cost mode (default `false`, key: `auto_nocost`): Automatically turns no cost mode on/off when devcommands are enabled or disabled. Requires automatic debug mode.
- Disable random events (default `false`, key: `disable_events`): Prevents random events from happening (server side setting).
- Invisible to players with ghost mode (default `false`, key: `ghost_invibisility`): Invisible to other players with ghost mode.
- No creature drops (default `false`, key: `no_drops`): Prevents creatures from dropping loot, can be useful if people accidentally spawn very high star creatures. Only works when as the zone owner.
- No knockback with god mode (default `true`, key: `god_no_knockback`): Removes knockback for an even godlier god mode.
- No staggering with god mode (default `true`, key: `god_no_stagger`): Removes staggering for an even godlier god mode.
- No stamina usage with god mode (default `true`, key: `god_no_stamina`): Removes stamina usage for an even godlier god mode.
- Show map coordinates (default `true`, key: `map_coordinates`): Shows cursor coordinates when hovering the map.
- Show private players (default `false`, key: `private_players`): Shows players on the map even if they have set their position as private. Must be enabled both client and server side to work (admins can individually keep the feature off even when enabled from the server). 

## Console

Recommended to keep all settings on default values, unless there are errors or mod conflicts.

- Alias system (default `true`, key: `aliasing`): Enables command aliasing.
- Auto exec (key: `auto_exec`): Executes the given command when joining a server (before admin is checked).
- Auto exec boot (key: `auto_exec_boot`): Executes the given command when starting the game.
- Auto exec dev off (key: `auto_exec_dev_off`): Executes the given command when disabling devcommands.
- Auto exec dev on (key: `auto_exec_dev_off`): Executes the given command when enabling devcommands.
- Command aliases: Saved command aliases.
- Command descriptions (default `true`, key: `command_descriptions`): Shows command descriptions as autocomplete.
- Debug console (default `false`, key: `debug_console`): Prints debug output to the console related to aliasing and parameter substitution.
- Delay between commands (key: `command_delay`): Adds delay (seconds) when executing multiple commands.
- Disable parameter warnings (default `false`, key: `disable_warnings`): Removes warning texts from some command parameter descriptions.
- Improved auto complete (default `true`, key: `improved_autocomplete`): Enables parameter info or options for every parameter.
- Multiple commands per line (default `true`, key: `multiple_commands`): Enables multiple commands per line (when separate by `;`).
- Substitution system (default `true`, key: `substitution`): Enables parameter substitution (with `$`).

# Changelog

- v1.10:
	- Adds a new setting `command_delay` to add delay when multiple commands are executed.
	- Renames the file from DEV.dll to ServerDevcommands.dll.
	- Changes the default value of the `ghost_invibisility` to false.
	- Fixes incompatibility with Mountain Caves public beta test.

- v1.9:
	- Adds a new command `resolution` to print or set screen properties.
	- Adds a new setting `command_descriptions` to show command descriptions instead of the autocomplete.
	- Adds a new setting `ghost_invibisility` to turn invisible to other players with the ghost mode.
	- Adds a new setting `god_no_knockback` to disable knockback with the god mode.
	- Adds a new setting `auto_exec_boot` that automatically executes the given command when starting the game.
	- Adds a new setting `auto_exec` that automatically executes the given command when joining a server.
	- Adds a new setting `auto_exec_dev_on` that automatically executes the given command when enabling `devcommands`.
	- Adds a new setting `auto_exec_dev_off` that automatically executes the given command when disabling `devcommands`.
	- Adds a parameter to command `tutorialtoggle` which directly sets the value (instead of toggling).
	- Changes the command `tutorialtoggle` to work without needing Hugin to appear first.
	- Changes the command `alias` to add the plain text as the description instead of the original command description.
	- Fixes case insensitivity being broken for some commands.
	- Fixes server side commands not working.

- v1.8:
	- Improves undo/redo system to work with Infinity Hammer mod.
	- Fixes selection not working when multiple commands system is on.
	- Fixes selection issues when using tab to cycle options.

- v1.7:
	- Adds an undo/redo system (currently only for other mods to use).
	- New icon (thanks Azumatt!).
	- Improves autocomplete for default commands.
	- Fixes possible server crash if private players are enabled (another attempt).

- v1.6:
	- Adds a better autocomplete that provides options and information for all parameters.
	- Adds an alias system which allows creating simpler commands out of existing ones.
	- Adds a parameter substitution system which allows mapping command parameters.
	- Adds support for multiple commands per line.
	- Adds a new command for setting server config values.
	- Adds a new command to search object ids.
	- Improves the admin check to support more features.
	- Adds a setting for automatic admin check (enabled by default).
	- Adds a setting for automatic debugmode.
	- Adds a setting for automatic god mode.
	- Adds a setting for automatic fly mode.
	- Adds a setting for automatic ghost mode.
	- Adds a setting for automatic no cost mode.
	- Adds a setting for improved autocomplete (enabled by default).
	- Adds a setting for command aliasing (enabled by default).
	- Adds a setting for command parameter substitution (enabled by default).
	- Adds a setting for multiple commands per line (enabled by default).
	- Adds a setting to remove stamina usage with god mode (enabled by default).
	- Adds a setting to remove staggering with god mode (enabled by default).
	- Adds a setting to disable creature drops.
	- Adds a setting to disable random events.
	- Changes the default value of "show private player positions" to false.
	- Fixes a server crash (caused by too many players connecting if private position feature was on).

- v1.5:
	- Adds a modifier key support to key bindings.
	- Adds a new parameter to the pos command (allows getting position of any player).
	- Adds support for showing private player positions (requires also server side).
	- Changes the setkey command to work client side.
	- Fixes console spam.
	- Fixes console commands not working in the character selection.
	- Attempts to further improve the admin check reliability.

- v1.4:
	- Adds server side support for event, randomevent, resetkeys, setkey, skiptime, sleep, stopevent.

- v1.3: 
	- Refactores the code to hopefully make it work more reliably.
	- Adds autocomplete to chat window also for cheat commands.

- v1.2: 
	- Adds support for Hearth and Home update.

- v1.1: 
	- Improves admin check.

- v1.0: 
	- Initial release.
	
Thanks for Azumatt for creating the mod icon!